@@ -37,13 +37,16 @@
 /************************************************
 
  ************************************************/
-CommandItemModel::CommandItemModel(QObject *parent) :
+CommandItemModel::CommandItemModel(bool useHistory, QObject *parent) :
     QSortFilterProxyModel(parent),
-    mSourceModel(new CommandSourceItemModel(this)),
-    mOnlyHistory(false)
+    mSourceModel(new CommandSourceItemModel(useHistory, this)),
+    mOnlyHistory(false),
+    mShowHistoryFirst(true)
 {
-	setDynamicSortFilter(false); // required in Qt5
+    setFilterCaseSensitivity(Qt::CaseInsensitive);
+    setSortCaseSensitivity(Qt::CaseInsensitive);
     setSourceModel(mSourceModel);
+    sort(0);
 }
 
 
@@ -81,11 +84,6 @@ const CommandProviderItem *CommandItemModel::command(const QModelIndex &index) c
 /************************************************
 
  ************************************************/
-void CommandItemModel::addHistoryCommand(const QString &command)
-{
-    mSourceModel->addHistoryCommand(command);
-}
-
 void CommandItemModel::clearHistory()
 {
     mSourceModel->clearHistory();
@@ -95,7 +93,7 @@ void CommandItemModel::clearHistory()
 /************************************************
 
  ************************************************/
-bool CommandItemModel::filterAcceptsRow(int sourceRow, const QModelIndex &/*sourceParent*/) const
+bool CommandItemModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
 {
     QRegExp re(filterRegExp());
 
@@ -107,7 +105,23 @@ bool CommandItemModel::filterAcceptsRow(int sourceRow, const QModelIndex &/*sour
     if (!item)
         return false;
 
-    return item->compare(re);
+    bool accept = item->compare(re);
+    if (accept)
+    {
+        //check if CustomCommand can be filtered out (equivalent app link is shown)
+        const CustomCommandItem * cust_i = qobject_cast<const CustomCommandItem *>(item);
+        if (nullptr != cust_i)
+        {
+            for (int i = mSourceModel->rowCount(sourceParent); 0 <= i; --i)
+            {
+                const AppLinkItem * app_i = qobject_cast<const AppLinkItem *>(mSourceModel->command(i));
+                if (nullptr != app_i && cust_i->exec() == app_i->exec() && app_i->compare(re))
+                    return false;
+            }
+        }
+    }
+
+    return accept;
 }
 
 
@@ -124,8 +138,25 @@ bool CommandItemModel::lessThan(const QModelIndex &left, const QModelIndex &righ
 
     if (mOnlyHistory)
         return left.row() < right.row();
-    else
-        return QSortFilterProxyModel::lessThan(left, right);
+
+    HistoryItem const * i_left = dynamic_cast<HistoryItem const *>(mSourceModel->command(left));
+    HistoryItem const * i_right = dynamic_cast<HistoryItem const *>(mSourceModel->command(right));
+    if (nullptr != i_left && nullptr == i_right)
+        return mShowHistoryFirst;
+    if (nullptr == i_left && nullptr != i_right)
+        return !mShowHistoryFirst;
+    if (nullptr != i_left && nullptr != i_right)
+    {
+        QRegExp re(filterRegExp());
+        //Note: -1 should not be returned if the item passed the filter previously
+        const int pos_left = re.indexIn(i_left->command());
+        const int pos_right = re.indexIn(i_right->command());
+        Q_ASSERT(-1 != pos_left && -1 != pos_right);
+        return pos_left < pos_right
+            || (pos_left == pos_right && QSortFilterProxyModel::lessThan(left, right));
+    }
+
+    return QSortFilterProxyModel::lessThan(left, right);
 }
 
 
@@ -138,7 +169,7 @@ int CommandItemModel::itemType(const QModelIndex &index) const
         return 1;
     else if (index.row() < mSourceModel->externalProviderStartIndex().row())
         return 2;
-    else 
+    else
         return 3;
 }
 
@@ -182,6 +213,15 @@ QModelIndex CommandItemModel::appropriateItem(const QString &pattern) const
 }
 
 
+/************************************************
+
+ ************************************************/
+void CommandItemModel::showHistoryFirst(bool first/* = true*/)
+{
+    mShowHistoryFirst = first;
+    invalidate();
+}
+
 /************************************************
 
  ************************************************/
@@ -194,17 +234,21 @@ void CommandItemModel::rebuild()
 /************************************************
 
  ************************************************/
-CommandSourceItemModel::CommandSourceItemModel(QObject *parent) :
-    QAbstractListModel(parent)
+CommandSourceItemModel::CommandSourceItemModel(bool useHistory, QObject *parent) :
+    QAbstractListModel(parent),
+    mHistoryProvider(nullptr)
 {
     mCustomCommandProvider = new CustomCommandProvider;
     mProviders.append(mCustomCommandProvider);
     rebuild();
     mCustomCommandIndex = index(0, 0);
 
-    mHistoryProvider = new HistoryProvider();
-    mProviders.append(mHistoryProvider);
-    mCustomCommandProvider->setHistoryProvider(mHistoryProvider);
+    if (useHistory)
+    {
+        mHistoryProvider = new HistoryProvider();
+        mProviders.append(mHistoryProvider);
+        mCustomCommandProvider->setHistoryProvider(mHistoryProvider);
+    }
 
     mProviders.append(new AppLinkProvider());
 #ifdef MATH_ENABLED
@@ -214,14 +258,12 @@ CommandSourceItemModel::CommandSourceItemModel(QObject *parent) :
     mProviders.append(new VirtualBoxProvider());
 #endif
 
-    mProviders.append(new PowerProvider());
-
     rebuild();
     mExternalProviderStartIndex = index(rowCount(), 0);
 
-    LxQt::Settings settings("lxqt-runner");
+    LXQt::Settings settings("lxqt-runner");
     int numExternalProviders = settings.beginReadArray("external providers");
-    for (int i = 0; i < numExternalProviders; i++) 
+    for (int i = 0; i < numExternalProviders; i++)
     {
         settings.setArrayIndex(i);
         qDebug() << "Adding external provider:" << settings.value("name") << settings.value("executable");
@@ -237,7 +279,7 @@ CommandSourceItemModel::CommandSourceItemModel(QObject *parent) :
         connect(provider, SIGNAL(changed()), this, SIGNAL(layoutChanged()));
         connect(provider, SIGNAL(aboutToBeChanged()), this, SIGNAL(layoutAboutToBeChanged()));
     }
-   
+
     rebuild();
 }
 
@@ -336,9 +378,16 @@ void CommandSourceItemModel::rebuild()
  ************************************************/
 void CommandSourceItemModel::clearHistory()
 {
-    beginResetModel();
-    mHistoryProvider->clearHistory();
-    endResetModel();
+    if (mHistoryProvider)
+    {
+        beginResetModel();
+        mHistoryProvider->clearHistory();
+        endResetModel();
+    } else
+    {
+        QScopedPointer<HistoryProvider> history_p{new HistoryProvider};
+        history_p->clearHistory();
+    }
 }
 
 
@@ -374,23 +423,41 @@ const CommandProviderItem *CommandSourceItemModel::command(const QModelIndex &in
 }
 
 
-/************************************************
-
- ************************************************/
-void CommandSourceItemModel::addHistoryCommand(const QString &command)
-{
-    mHistoryProvider->AddCommand(command);
-}
-
-
 /***********************************************
- 
+
  ***********************************************/
 void CommandSourceItemModel::setCommand(const QString& command)
 {
-    mCustomCommandProvider->setCommand(command); 
+    mCustomCommandProvider->setCommand(command);
     foreach (ExternalProvider* externalProvider, mExternalProviders)
     {
         externalProvider->setSearchTerm(command);
     }
 }
+
+
+/***********************************************
+
+ ***********************************************/
+void CommandSourceItemModel::setUseHistory(bool useHistory)
+{
+    const bool now_using_history = mHistoryProvider != nullptr;
+    if (now_using_history == useHistory)
+        return;
+    beginResetModel();
+    if (now_using_history)
+    {
+        mProviders.removeAll(mHistoryProvider);
+        mCustomCommandProvider->setHistoryProvider(nullptr);
+        delete mHistoryProvider;
+        mHistoryProvider = nullptr;
+    } else
+    {
+        mHistoryProvider = new HistoryProvider;
+        mProviders.append(mHistoryProvider);
+        mCustomCommandProvider->setHistoryProvider(mHistoryProvider);
+        connect(mHistoryProvider, SIGNAL(changed()), this, SIGNAL(layoutChanged()));
+        connect(mHistoryProvider, SIGNAL(aboutToBeChanged()), this, SIGNAL(layoutAboutToBeChanged()));
+    }
+    endResetModel();
+}
