@@ -51,7 +51,7 @@
 #include <QWindow>
 #include <QScrollBar>
 
-#include <KF5/KWindowSystem/KWindowSystem>
+#include <KWindowSystem/KWindowSystem>
 
 #define DEFAULT_SHORTCUT "Alt+F2"
 
@@ -59,18 +59,19 @@
 
  ************************************************/
 Dialog::Dialog(QWidget *parent) :
-    QDialog(parent, Qt::Tool | Qt::WindowStaysOnTopHint | Qt::CustomizeWindowHint),
+    QDialog(parent, Qt::Dialog | Qt::WindowStaysOnTopHint | Qt::CustomizeWindowHint),
     ui(new Ui::Dialog),
-    mSettings(new LxQt::Settings("lxqt-runner", this)),
+    mSettings(new LXQt::Settings("lxqt-runner", this)),
     mGlobalShortcut(0),
     mLockCascadeChanges(false),
+    mDesktopChanged(false),
     mConfigureDialog(0)
 {
     ui->setupUi(this);
     setWindowTitle("LXQt Runner");
     setAttribute(Qt::WA_TranslucentBackground);
 
-    connect(LxQt::Settings::globalSettings(), SIGNAL(iconThemeChanged()), this, SLOT(update()));
+    connect(LXQt::Settings::globalSettings(), SIGNAL(iconThemeChanged()), this, SLOT(update()));
     connect(ui->closeButton, SIGNAL(clicked()), this, SLOT(hide()));
     connect(mSettings, SIGNAL(settingsChanged()), this, SLOT(applySettings()));
 
@@ -79,15 +80,15 @@ Dialog::Dialog(QWidget *parent) :
     connect(ui->commandEd, SIGNAL(textChanged(QString)), this, SLOT(setFilter(QString)));
     connect(ui->commandEd, SIGNAL(returnPressed()), this, SLOT(runCommand()));
 
-    mCommandItemModel = new CommandItemModel(this);
+    mCommandItemModel = new CommandItemModel(mSettings->value("dialog/history_use", true).toBool(), this);
     ui->commandList->installEventFilter(this);
     ui->commandList->setModel(mCommandItemModel);
     ui->commandList->setEditTriggers(QAbstractItemView::NoEditTriggers);
     connect(ui->commandList, SIGNAL(clicked(QModelIndex)), this, SLOT(runCommand()));
-    setFilter("");
+    setFilter(QString());
     dataChanged();
 
-    ui->commandList->setItemDelegate(new LxQt::HtmlDelegate(QSize(32, 32), ui->commandList));
+    ui->commandList->setItemDelegate(new LXQt::HtmlDelegate(QSize(32, 32), ui->commandList));
 
     // Popup menu ...............................
     QAction *a = new QAction(XdgIcon::fromTheme("configure"), tr("Configure"), this);
@@ -98,9 +99,9 @@ Dialog::Dialog(QWidget *parent) :
     connect(a, SIGNAL(triggered()), mCommandItemModel, SLOT(clearHistory()));
     addAction(a);
 
-    mPowerManager = new LxQt::PowerManager(this);
+    mPowerManager = new LXQt::PowerManager(this);
     addActions(mPowerManager->availableActions());
-    mScreenSaver = new LxQt::ScreenSaver(this);
+    mScreenSaver = new LXQt::ScreenSaver(this);
     addActions(mScreenSaver->availableActions());
 
     setContextMenuPolicy(Qt::ActionsContextMenu);
@@ -108,7 +109,6 @@ Dialog::Dialog(QWidget *parent) :
     QMenu *menu = new QMenu(this);
     menu->addActions(actions());
     ui->actionButton->setMenu(menu);
-    ui->actionButton->setIcon(XdgIcon::fromTheme("configure"));
     // End of popup menu ........................
 
     applySettings();
@@ -117,6 +117,8 @@ Dialog::Dialog(QWidget *parent) :
     connect(QApplication::desktop(), SIGNAL(workAreaResized(int)), SLOT(realign()));
     connect(mGlobalShortcut, SIGNAL(activated()), this, SLOT(showHide()));
     connect(mGlobalShortcut, SIGNAL(shortcutChanged(QString,QString)), this, SLOT(shortcutChanged(QString,QString)));
+    connect(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)), this, SLOT(onActiveWindowChanged(WId)));
+    connect(KWindowSystem::self(), &KWindowSystem::currentDesktopChanged, this, &Dialog::onCurrentDesktopChanged);
 
     resize(mSettings->value("dialog/width", 400).toInt(), size().height());
 
@@ -140,7 +142,7 @@ Dialog::~Dialog()
 void Dialog::closeEvent(QCloseEvent *event)
 {
     hide();
-    event->ignore();
+    event->accept();
 }
 
 
@@ -179,11 +181,6 @@ bool Dialog::eventFilter(QObject *object, QEvent *event)
         if (object == ui->commandList)
             return listKeyPressEvent(keyEvent);
     }
-    else if (event->type() == QEvent::FocusOut)
-    {
-        hide();
-        return true;
-    }
 
     return QDialog::eventFilter(object, event);
 }
@@ -221,7 +218,7 @@ bool Dialog::editKeyPressEvent(QKeyEvent *event)
             ui->commandList->currentIndex().row() == 0
            )
         {
-            setFilter("", false);
+            setFilter(QString(), false);
             return true;
         }
         qApp->sendEvent(ui->commandList, event);
@@ -233,10 +230,14 @@ bool Dialog::editKeyPressEvent(QKeyEvent *event)
             ui->commandList->isHidden()
            )
         {
-            setFilter("", true);
+            setFilter(QString(), true);
+
+            // set focus to the list so that it highlights the first item correctly,
+            // and then set it back to the textfield, where it belongs
+            ui->commandList->setFocus();
+            ui->commandEd->setFocus();
             return true;
         }
-
         qApp->sendEvent(ui->commandList, event);
         return true;
 
@@ -292,7 +293,9 @@ bool Dialog::listKeyPressEvent(QKeyEvent *event)
  ************************************************/
 void Dialog::showHide()
 {
-    if (isVisible() && isActiveWindow())
+    // Using KWindowSystem to detect the active window since
+    // QWidget::isActiveWindow is not working reliably.
+    if (isVisible() && (KWindowSystem::activeWindow() == winId()))
     {
         hide();
     }
@@ -300,7 +303,7 @@ void Dialog::showHide()
     {
         realign();
         show();
-        KWindowSystem::forceActiveWindow(windowHandle()->winId());
+        KWindowSystem::forceActiveWindow(winId());
         ui->commandEd->setFocus();
     }
 }
@@ -353,6 +356,10 @@ void Dialog::applySettings()
 
     mMonitor = mSettings->value("dialog/monitor", -1).toInt();
 
+    mCommandItemModel->setUseHistory(mSettings->value("dialog/history_use", true).toBool());
+    mCommandItemModel->showHistoryFirst(mSettings->value("dialog/history_first", true).toBool());
+    ui->commandList->setShownCount(mSettings->value("dialog/list_shown_items", 4).toInt());
+
     realign();
     mSettings->sync();
 }
@@ -375,20 +382,60 @@ void Dialog::shortcutChanged(const QString &/*oldShortcut*/, const QString &newS
 }
 
 
+/************************************************
+
+ ************************************************/
+void Dialog::onActiveWindowChanged(WId id)
+{
+    if (isVisible() && 0 != id && id != winId())
+    {
+        if (mDesktopChanged)
+        {
+            mDesktopChanged = false;
+            KWindowSystem::forceActiveWindow(winId());
+        } else
+        {
+            hide();
+        }
+    }
+}
+
+
+/************************************************
+
+ ************************************************/
+void Dialog::onCurrentDesktopChanged(int screen)
+{
+    if (isVisible())
+    {
+        KWindowSystem::setOnDesktop(winId(), screen);
+        KWindowSystem::forceActiveWindow(winId());
+        //Note: workaround for changing desktop while runner is shown
+        // The KWindowSystem::forceActiveWindow may fail to correctly activate runner if there
+        // are any other windows on the new desktop (probably because of the sequence while WM
+        // changes the virtual desktop (change desktop and activate any of the windows on it))
+        mDesktopChanged = true;
+    }
+}
+
+
 /************************************************
 
  ************************************************/
 void Dialog::setFilter(const QString &text, bool onlyHistory)
 {
-    qDebug() << "Ind i setFilter...";
     if (mCommandItemModel->isOutDated())
         mCommandItemModel->rebuild();
 
     QString trimmedText = text.simplified();
     mCommandItemModel->setCommand(trimmedText);
     mCommandItemModel->showOnlyHistory(onlyHistory);
-    mCommandItemModel->setFilterWildcard(trimmedText);
-    mCommandItemModel->sort(0);
+    mCommandItemModel->setFilterRegExp(trimmedText);
+    mCommandItemModel->invalidate();
+
+    // tidy up layout and select first item
+    ui->commandList->doItemsLayout();
+    ui->commandList->setCurrentIndex(mCommandItemModel->index(0, 0));
 }
 
 /************************************************
