@@ -26,18 +26,32 @@
  * END_COMMON_COPYRIGHT_HEADER */
 
 
-#include <LXQt/Application>
+#include <LXQt/SingleApplication>
+
+#include <QCommandLineParser>
+
 #include "dialog.h"
 
 
 int main(int argc, char *argv[])
 {
-    LxQt::Application a(argc, argv);
+    LXQt::SingleApplication a(argc, argv);
     a.setQuitOnLastWindowClosed(false);
+    a.setAttribute(Qt::AA_UseHighDpiPixmaps, true);
 
-    QWidget *hiddenPreviewParent = new QWidget(0, Qt::Tool);
-    Dialog d(hiddenPreviewParent);
-    //d.show();
+    QCommandLineParser parser;
+    parser.setApplicationDescription(QStringLiteral("LXQt Runner"));
+    const QString VERINFO = QStringLiteral(LXQT_RUNNER_VERSION
+                                           "\nliblxqt   " LXQT_VERSION
+                                           "\nQt        " QT_VERSION_STR);
+    a.setApplicationVersion(VERINFO);
+    parser.addVersionOption();
+    parser.addHelpOption();
+    parser.process(a);
+
+    QWidget hiddenPreviewParent{0, Qt::Tool};
+    Dialog d(&hiddenPreviewParent);
+    a.setActivationWindow(&d);
 
     return a.exec();
 
